/**********************************************************************
**  Copyright 2003 Craig Graham <craig@data-uncertain.co.uk>
**  Copyright 2003 Steve Essery <tecknobabble@hotmail.com>
**
**  This file is part of the SubApplet Applet.
**
**  The SubApplet Applet is free software; you can redistribute it
**  and/or modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2 of
**  the License, or (at your option) any later version.
**
**  The SubApplet Applet is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with the SubApplet Applet; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**********************************************************************/
#ifndef _ABOUTFORM_H_
#include "aboutform.h"
#endif

#include "button_ok.xpm"
#include "button_cancel.xpm"

#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qdir.h>
#include <qslider.h>
#include <qpe/config.h>
#include <qpe/qpeapplication.h>
#include <qlabel.h>
#ifdef QWS
#include <qpe/qcopenvelope_qws.h>
#endif


AboutForm::AboutForm(SubApplet *subapplet, QWidget *parent,
                     const char *name, bool modal)
  : AboutFormBase(NULL, name, modal), subApplet(subapplet),
    mResyncConfig(true), mUpdateMenuTab(true)
{
    mDebugHelper = DebugHelper("/tmp/aboutform.log");

    resyncConfig();

    mDebugHelper.logMessage("AboutForm: entered");
    
    // Connect the Tab selected signal to the routine use to ensure the Menu
    // and Config Tabs are displaying the correct contents
    connect(tabWidget, SIGNAL(currentChanged(QWidget*)),
            this, SLOT(setupTabContents(QWidget*)));

    // Signals for the checkboxes
    connect(drawButtonCheckbox, SIGNAL(clicked()),
            this, SLOT(drawCheckBoxChanged()));
    connect(flatButtonCheckbox, SIGNAL(clicked()),
            this, SLOT(flatCheckBoxChanged()));
    connect(debugCheckbox, SIGNAL(clicked()),
            this, SLOT(debugCheckBoxChanged()));

    // Signal for the position spinbox
    connect(positionSpinBox, SIGNAL(valueChanged(int)),
            this, SLOT(positionChanged(int)));

    // Hackette as the setMinValue doesn't appear to like 0...odd
    positionSpinBox->setRange(0,10);

    // Need to pick up when a row in the list view is clicked
    connect(menuListView, SIGNAL(clicked(QListViewItem *)),
            this, SLOT(rowClicked(QListViewItem *)));

    // Need to update the list when a menu name is changed
    connect(nameOkButton, SIGNAL(clicked()),
            this, SLOT(nameChanged()));

    // But don't want the button to be enabled until they have selected a row
    nameOkButton->setEnabled(false);

    // Tie the move up button to its slot
    connect(moveUpButton, SIGNAL(clicked()),
            this, SLOT(moveRowUp()));

    // Tie the move down button to its slot
    connect(moveDownButton, SIGNAL(clicked()),
            this, SLOT(moveRowDown()));

    // Tie the slider signal to our handler
    connect(popupDelaySlider, SIGNAL(valueChanged(int)),
            this, SLOT(sliderChanged(int)));

    // Disable sorting in the list view
    menuListView->setSorting(-1);

    // Setup the toggleButton
    mOkPixmap = QPixmap(button_ok_xpm);
    mCancelPixmap = QPixmap(button_cancel_xpm);
    toggleButton->setText("");
    toggleButton->setToggleButton(TRUE);
    toggleButton->setOn(TRUE);

    // Tie things up
    connect(toggleButton, SIGNAL(toggled(bool)),
            this, SLOT(toggleClicked(bool)));

    mDebugHelper.logMessage("AboutForm: exited");
}


AboutForm::~AboutForm()
{
    mDebugHelper.logMessage("~AboutForm: entered");
    mDebugHelper.logMessage("~AboutForm: exited");
    mDebugHelper.setDebugFlag(false);
}


void AboutForm::checkGeometry(bool subappletVisible)
{
    mDebugHelper.logMessage("checkGeometry: entered");

    int rotation = QPEApplication::defaultRotation();
    int dwidth   = QPEApplication::desktop()->width();
    int dheight  = QPEApplication::desktop()->height();

    // Work out how big we should be
    if (mDebugHelper.debugFlag())
    {
        mDebugHelper.logMessage("checkGeometry: desktop width=%d", dwidth);
        mDebugHelper.logMessage("checkGeometry: desktop height=%d", dheight);
        mDebugHelper.logMessage("checkGeometry: desktop rotation=%d", rotation);
    }

    resize(dwidth, dheight - (subappletVisible ? 40 : 20 )); 

    updateGeometry();

    mDebugHelper.logMessage("checkGeometry: exited");
}


void AboutForm::moveRowUp()
{
    mDebugHelper.logMessage("moveRowUp: entered");

    // Find the ListViewItem that is selected
    QListViewItem *rowToMove = menuListView->selectedItem();

    // Don't do anything if this is null
    if (rowToMove != NULL)
    {
	// Find the item above us
        QListViewItem *itemAbove = rowToMove->itemAbove();

	if (itemAbove != NULL)
	{
	    // If itemAbove is null then we must be at the top already, but
	    // if itemAbove is not null then see if we can find the item above
	    // it.
	    QListViewItem *itemAboveItemAbove = itemAbove->itemAbove();

	    if (itemAboveItemAbove == NULL)
	    {
	        // If this is null then the item above the row we are trying to
		// move is the top item, in which case move it under us
		itemAbove->moveItem(rowToMove);
	    }
	    else
	    {
	        // If the itemAbove the itemAbove is not null we can move the
		// row we want to move to under it
		rowToMove->moveItem(itemAboveItemAbove);
	    }
	    menuListView->ensureItemVisible(rowToMove);
	    menuListView->triggerUpdate();
	}
    }

    mDebugHelper.logMessage("moveRowUp: exited");
}


void AboutForm::moveRowDown()
{
    mDebugHelper.logMessage("moveRowDown: entered");

    // Find the ListViewItem that is selected
    QListViewItem *rowToMove = menuListView->selectedItem();

    // Don't do anything if this is null
    if (rowToMove != NULL)
    {
        // Find the item below us
	QListViewItem *itemBelow = rowToMove->itemBelow();

	if (itemBelow != NULL)
	{
	    // If itemBelow is null then the row we are trying to move must be
	    // at the bottom already.
	    // If itemBelow is not null then we can simply move ourselves
	    // after it - so much easier than the moving up case :-)
	    rowToMove->moveItem(itemBelow);
	    menuListView->ensureItemVisible(rowToMove);
	    menuListView->triggerUpdate();
	}
    }

    mDebugHelper.logMessage("moveRowDown: exited");
}


void AboutForm::nameChanged()
{
    mDebugHelper.logMessage("nameChanged: entered");

    // Find the ListViewItem that is still selected
    QListViewItem *lvi = menuListView->selectedItem();

    // Get the new text
    QString newName = nameEdit->text();

    // Disallow a blank name
    if (!newName.isEmpty())
    {
	// Update the Menu Name column for the item
	lvi->setText(1, nameEdit->text());
    }

    // Clear the selection
    menuListView->clearSelection();

    // Disable the ok button
    nameOkButton->setEnabled(false);

    // Clear the nameEdit field
    nameEdit->clear();

    // Disable the toggle button
    toggleButton->setEnabled(false);
    toggleButton->setText("");

    mDebugHelper.logMessage("nameChanged: exited");
}


void AboutForm::toggleClicked(bool state)
{
    mDebugHelper.logMessage("toggleClicked: entered");

    mDebugHelper.logMessage("toggleClicked: state=%d", state);

    QListViewItem *lvi = menuListView->currentItem();
    if (lvi != NULL)
    {
	mDebugHelper.logMessage("toggleClicked: row selected in list view");
        if (state)
	{
	    toggleButton->setPixmap(mOkPixmap);
	    QString lib = lvi->text(0);
	    lvi->setPixmap(0, mOkPixmap);
	    mExcludeApplets.remove(lib);
	    mDebugHelper.logMessage("toggleClicked: removing %s from exclude list", lib.latin1());
	}
	else
	{
	    toggleButton->setPixmap(mCancelPixmap);
	    QString lib = lvi->text(0);
	    lvi->setPixmap(0, mCancelPixmap);
	    mExcludeApplets.append(lib);
	    mDebugHelper.logMessage("toggleClicked: appending %s to exclude list", lib.latin1());
	}
    }
    else
    {
	mDebugHelper.logMessage("toggleClicked: no row selected, disabling toggle");
	toggleButton->setText("");
	toggleButton->setEnabled(false);
    }

    mDebugHelper.logMessage("toggleClicked: exited");
}


void AboutForm::rowClicked(QListViewItem *row)
{
    mDebugHelper.logMessage("rowClicked: entered");

    if (row)
    {
	mOrigName = row->text(1);
	nameEdit->setText(mOrigName);
	nameOkButton->setEnabled(true);

	if (mExcludeApplets.findIndex(row->text(0)) == -1)
	{
	    mDebugHelper.logMessage("rowClicked: applet not on exclude list");
	    toggleButton->setOn(true);
	    toggleButton->setPixmap(mOkPixmap);
	}
	else
	{
	    mDebugHelper.logMessage("rowClicked: applet on exclude list");
	    toggleButton->setOn(false);
	    toggleButton->setPixmap(mCancelPixmap);
	}
	toggleButton->setEnabled(true);
    }
    else
    {
	toggleButton->setText("");
	toggleButton->setEnabled(false);
	nameEdit->clear();
    }

    mDebugHelper.logMessage("rowClicked: exited");
}


void AboutForm::drawCheckBoxChanged()
{
    mDebugHelper.logMessage("drawCheckBoxChanged: entered");
    mUseDrawButton = drawButtonCheckbox->isChecked(); 
    flatButtonCheckbox->setEnabled(mUseDrawButton);
    mDebugHelper.logMessage("drawCheckBoxChanged: exited");
}


void AboutForm::flatCheckBoxChanged()
{
    mDebugHelper.logMessage("flatCheckBoxChanged: entered");
    mUseFlatButton = flatButtonCheckbox->isChecked();
    mDebugHelper.logMessage("flatCheckBoxChanged: exited");
}


void AboutForm::debugCheckBoxChanged()
{
    mDebugHelper.logMessage("debugCheckBoxChanged: entered");
    mUseDebug      = debugCheckbox->isChecked();
    mDebugHelper.logMessage("debugCheckBoxChanged: exited");
}


void AboutForm::positionChanged(int val)
{
    mDebugHelper.logMessage("positionChanged: entered");
    mPosition = val;
    mDebugHelper.logMessage("positionChanged: exited");
}


void AboutForm::setupTabContents(QWidget *)
{
    mDebugHelper.logMessage("setupTabContents: entered");

    int id = tabWidget->currentPageIndex();
    mDebugHelper.logMessage("setupTabContents: id=%d", id);

    if (mResyncConfig)
    {
        resyncConfig();
    }

    // ID 0 = Menu Tab
    if (id == 0)
    {
        if (mUpdateMenuTab)
	{
	    // Clear the list view
	    menuListView->clear();

	    // Interestingly the items are displayed in the reverse order
	    // in which they are created in the list view so do some more
	    // work to get things the way we want.  Also protects us if 
	    // the internal workings of the QListView change in another
	    // release.

	    QStringList::Iterator libIt = mLibNames.begin();
	    QStringList::Iterator nameIt = mNiceNames.begin();

	    QListViewItem *lastItem = menuListView->firstChild();
	    bool first = true;

	    while (libIt != mLibNames.end() && nameIt != mNiceNames.end())
	    {
		mDebugHelper.logMessage("setupTabContents: adding %s=>%s to view", (*libIt).latin1(), (*nameIt).latin1());
	        if (first)
		{
		    new QListViewItem(menuListView, *libIt, *nameIt);
		    lastItem = menuListView->firstChild();
		    first = false;
		}
		else
		{
		    new QListViewItem(menuListView, lastItem, *libIt, *nameIt);
		    lastItem = lastItem->itemBelow();
		}

		if (mExcludeApplets.findIndex(*libIt) == -1)
		{
		    lastItem->setPixmap(0, mOkPixmap);
		}
		else
		{
		    lastItem->setPixmap(0, mCancelPixmap);
		}

		libIt++;
		nameIt++;
	    }

	    menuListView->triggerUpdate();

	    mUpdateMenuTab = false;
	}
    }

    // ID 1 = Config Tab
    if (id == 1)
    {
        // Note we are using the copies of the values we copied when first 
	// opened.  Will only write out the configuration if the dialogue is
	// closed with the ok button
        drawButtonCheckbox->setChecked(mUseDrawButton);
	flatButtonCheckbox->setChecked(mUseFlatButton);
	debugCheckbox->setChecked(mUseDebug);
	positionSpinBox->setValue(mPosition);
	popupDelaySlider->setValue(mPopupDelay);
    }

    mDebugHelper.logMessage("setupTabContents: exited");
}


void AboutForm::sliderChanged(int value)
{
    popupDelayValueLabel->setNum(value);
    mPopupDelay = value;
}


void AboutForm::accept()
{
    mDebugHelper.logMessage("accept: entered");

    mResyncConfig = true;
    mUpdateMenuTab = true;

    // Write out the updated information
    QDir settingsDir(QDir::homeDirPath()+"/Settings");
    QString configFilePath(settingsDir.filePath("SubApplet.conf"));
    Config *cfgFile = new Config(configFilePath, Config::File);
    cfgFile->setGroup("Settings");

    // Write out the config parameters
    cfgFile->writeEntry("DrawButton", mUseDrawButton);
    cfgFile->writeEntry("FlatButton", mUseFlatButton);
    cfgFile->writeEntry("Debug",      mUseDebug);
    cfgFile->writeEntry("Position",   mPosition);
    cfgFile->writeEntry("PopupDelay", popupDelaySlider->value());

    // Write out the exclude list
    cfgFile->writeEntry("ExcludeApplets", mExcludeApplets, ',');
    
    // Build up the library and name lists from the ListView
    QStringList libList;
    QStringList nameList;

    QListViewItem *lvi = menuListView->firstChild();
    while (lvi != NULL)
    {
	libList  += lvi->text(0);
	nameList += lvi->text(1);
	lvi = lvi->itemBelow();
    }
    cfgFile->writeEntry("FileLibNames", libList, ',');
    cfgFile->writeEntry("DisplayLibNames", nameList, ',');
    delete cfgFile;

    menuListView->clear();
    hide();

    // Make sure the SubApplet knows about the updates
    subApplet->readConfig();

    // Fire a QCop message
    QCopChannel::send("QPE/TaskBar", "reloadApplets()");

    mDebugHelper.logMessage("accept: exited");
}


void AboutForm::reject()
{
    mDebugHelper.logMessage("reject: entered");

    resyncConfig();
    mResyncConfig = true;
    mUpdateMenuTab = true;
    menuListView->clear();

    mDebugHelper.logMessage("reject: exited");
}


void AboutForm::resyncConfig()
{
    mDebugHelper.logMessage("resyncConfig: entered");

    if (mResyncConfig)
    {
	// Read in the subapplet config to make sure things are uptodate
	if (subApplet)
	{
	    subApplet->readConfig();

	    // Make local copies of the configuration values
	    mUseDrawButton  = subApplet->useDrawButton();
	    mUseFlatButton  = subApplet->useFlatButton();
	    mUseDebug       = subApplet->useDebug();
	    mPosition       = subApplet->position();
	    mLibNameDict    = subApplet->libNameDict();
	    mNiceNameDict   = subApplet->niceNameDict();
	    mLibNames       = subApplet->libNames();
	    mNiceNames      = subApplet->niceNames();
	    mExcludeApplets = subApplet->excludeApplets();
	    mPopupDelay     = subApplet->delay();
	}

	mDebugHelper.setDebugFlag(mUseDebug);
	mResyncConfig = false;
    }

    mDebugHelper.logMessage("resyncConfig: exited");
}


/*
void AboutForm::showEvent(QShowEvent *)
{
    mDebugHelper.logMessage("showEvent: entered");

    resyncConfig();

    mDebugHelper.logMessage("showEvent: exited");
}
*/
