/**********************************************************************
**  Copyright 2003 Craig Graham <craig@data-uncertain.co.uk>
**  Copyright 2003 Steve Essery <tecknobabble@hotmail.com>
**
**  This file is part of the SubApplet Applet.
**
**  The SubApplet Applet is free software; you can redistribute it
**  and/or modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2 of
**  the License, or (at your option) any later version.
**
**  The SubApplet Applet is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with the SubApplet Applet; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**********************************************************************/
#ifndef _SUBAPPLET_H_
#define _SUBAPPLET_H_

#include "debug.h"

#include <qwidget.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qstringlist.h>
#include <qguardedptr.h>
#include <qdict.h>
#ifdef QWS
#include <qcopchannel_qws.h>
#endif
#include <qtoolbutton.h>

class AboutForm;
class QLibrary;
class TaskbarAppletInterface;
class QHBoxLayout;

typedef QDict<QString> StringDict;

typedef struct
{
#ifndef QT_NO_COMPONENT
    QLibrary *library;
#endif
    TaskbarAppletInterface *iface;
    QWidget *applet;
} TaskbarApplet;

class SubApplet : public QToolButton
{
    Q_OBJECT
public:
    SubApplet( QWidget *parent );
    ~SubApplet( );
   
    void unloadApplets();
    void loadApplets();
    void readConfig();

    bool useDrawButton();
    bool useFlatButton();
    bool useDebug();
    int  position();
    int  delay();

    void setUseDrawButton(bool);
    void setUseFlatButton(bool);
    void setUseDebug(bool);
    void setPosition(int);
    void setDelay(int);

    StringDict  libNameDict();
    StringDict  niceNameDict();
    QStringList libNames();
    QStringList niceNames();
    QStringList excludeApplets();


protected slots:
    void showAbout();
    void menuItemClicked(int);
    void heldDown();
    void fireReload();
#ifdef QWS
    void signalReceived(const QCString&, const QByteArray& data);
#endif

protected:
    void mousePressEvent( QMouseEvent * );
    void mouseReleaseEvent( QMouseEvent * );
#ifdef QWS
    QCopChannel *m_pChannel;
#endif

private:
    void paintEvent( QPaintEvent* );
	
    QPixmap                    mPixmapOpen;
    QPixmap                    mPixmapClosed;
    QPixmap                    mPixmapInfo;
    QGuardedPtr<QWidget>       m_pChild;
    QGuardedPtr<AboutForm>     m_pAbout;
    QGuardedPtr<QHBoxLayout>   m_pLayout;
    QGuardedPtr<QPopupMenu>    m_pMenu;
    QValueList<TaskbarApplet*> m_appletList;
    bool                       mButtonDown;
    QGuardedPtr<QTimer>        m_pTimer;
    StringDict                 mLibNameDict;
    StringDict                 mNiceNameDict;
    QStringList                mLibNames;            
    QStringList                mNiceNames;
    QStringList                mExcludeApplets;
    bool                       mTaskBarConfFound;
    bool                       mUseDrawButton;
    bool                       mUseFlatButton;
    bool                       mUseDebug;
    int                        mPosition;
    int                        mPopupDelay;
    DebugHelper                mDebugHelper;
};


#endif // _SUBAPPLET_H_
